
#include "event_emitter.h"

#define EventsIterator map<string, list<EventEmitter::EventListener*>>::iterator
#define ListenersIterator list<EventEmitter::EventListener*>::iterator

EventEmitter::EventListener::EventListener(const Local<Function>& callback, const bool once){
    this->callback = new Nan::Callback(callback);
    this->once = once;
}

EventEmitter::EventListener::~EventListener() {
    Release();
}

void EventEmitter::EventListener::Release() {
    HI_DELETE(callback);
}

EventEmitter::EventEmitter(node::ObjectWrap *parent) {
    this->_parent = parent;
}

EventEmitter::~EventEmitter() {
    for (EventsIterator list_it= _events.begin(); list_it != _events.end(); list_it ++ ){
        for(ListenersIterator it = list_it->second.begin(); it != list_it->second.end(); it++){
            (*it)->Release();
        }
        list_it->second.clear();
    }
    _events.clear();
}

bool EventEmitter::On(const string& type, const Local<Value>& callback, const bool once) {
    if (callback->IsFunction()) {
        _events[type].push_back( new EventListener(callback.As<Function>() , once) );
        return true;
    }
    return false;
}

void EventEmitter::Un(const string& type, const Local<Function>& callback) {
    if (callback.IsEmpty()) {
        return;
    }
    EventsIterator list_it = _events.find(type);
    if (list_it == _events.end()) {
        return;
    }
    list<EventListener*>* ls = &list_it->second;
    if (ls != NULL && !ls->empty()){
        for(ListenersIterator it = ls->begin(); it != ls->end(); it++){
            auto ref = *it;
            if (ref->callback == NULL || ref->callback->GetFunction() == callback ) {
                ref->Release();
                ls->erase(it);
            }
        }
    }
}

void EventEmitter::Emit(const string& type, int argc, Local<Value> argv[]) {
    if (_parent == NULL) {
        return;
    }
    EventsIterator list_it = _events.find(type);
    if (list_it == _events.end()) {
        return;
    }
    list<EventListener*>* ls = &list_it->second;
    if (ls != NULL && !ls->empty()){
        HI_SCOPE;
        Local<Object> handle = _parent->handle();
        for(ListenersIterator it = ls->begin(); it != ls->end(); it++){
            auto ref = *it;
            if (ref->callback != NULL && !ref->callback->IsEmpty()) {
                ref->callback->Call(handle, argc, argv);
            }
            if (ref->callback == NULL || ref->once ) {
                ref->Release();
                ls->erase(it);
            }
        }
    }
}

bool EventEmitter::HasListener(const string& type) {
    if (_parent == NULL) {
        return false;
    }
    EventsIterator it = _events.find(type);
    if (it == _events.end()) {
        return false;
    }
    if ( it->second.empty() ) {
        return false;
    }
    return true;
}
