
#ifndef hi_node_h
#define hi_node_h

#include <node.h>
#include <nan.h>
#include <string>
using namespace v8;
using namespace std;

#define HI_DELETE(variable) \
if (variable != NULL) { delete variable; variable = NULL;}

// -> HI_SCOPE
#define HI_SCOPE \
Isolate *isolate = Isolate::GetCurrent();\
HandleScope scope(isolate)

// -> HI_ISOLATE
#define HI_ISOLATE \
Isolate *isolate = Isolate::GetCurrent()

#define HI_BOOL(V, DEF) \
(!V->IsBoolean() ? V->BooleanValue() : DEF)

#define HI_BOOLEAN(bool) \
(bool ? True(isolate) : False(isolate))

// -> HI_STRING
#define HI_TO_STR(value) \
string(*String::Utf8Value(value->IsString()?value.As<String>():value->ToString()))

#define HI_STR(value) \
(value->IsString() ? string(*String::Utf8Value(value.As<String>())) : string())

// -> HI_STRING
#define HI_STRING(str) \
HI::ToString(str)

// -> HI_FUNCTION
#define HI_FUNCTION(fn) \
FunctionTemplate::New(isolate, fn)->GetFunction()

#define HI_MODULE(name) \
void Module_Init(Handle<Object> exports, Handle<Object> module);\
NODE_MODULE(name, Module_Init);\
void Module_Init(Handle<Object> exports, Handle<Object> module)


////////////////////////////////////////////////////////////////////////////

#define HI_UNWRAP(OBJ, value) \
node::ObjectWrap::Unwrap<OBJ>(value)

// -> HI_ARGS_INFO
#define HI_ARGS_INFO \
const FunctionCallbackInfo<Value>& info

#define HI_ARGS_PROPERTY_INFO \
const PropertyCallbackInfo<void>& info

// -> HI_FUNC
#define HI_METHOD(name) \
void name(HI_ARGS_INFO)

// -> HI_OBJ_GETTER
#define HI_GETTER(name) \
void name(Local<String> property, const PropertyCallbackInfo<Value>& info)

// -> HI_OBJ_SETTER
#define HI_SETTER(name) \
void name(Local<String> property, Local<Value> value, const PropertyCallbackInfo<void>& info)


// -> HI_FUNC_RETURN
#define HI_RETURN(value) \
info.GetReturnValue().Set(value);\
return

// -> HI_FUNC_RETURN
#define HI_RETURN_VALUE(value) \
info.GetReturnValue().Set(value);

// -> HI_RETURN_THIS
#define HI_RETURN_THIS \
info.GetReturnValue().Set(info.This());\
return

// -> HI_NAN_FUNCTION
#define HI_NAN_FUNCTION(fn) \
Nan::New<FunctionTemplate>(fn)->GetFunction()

// -> HI_UNDEFINED
#define HI_UNDEFINED \
Undefined(isolate)


////////////////////////////////////////////////////////////////////////////
// -> HI_CLASS
#define HI_CLASS(CLS)\
CLS : public node::ObjectWrap

// -> HI_CLASS_DEFINE
#define HI_CLASS_DEFINE(CLS)\
    CLS(HI_ARGS_INFO);\
    static Persistent<FunctionTemplate> Template;\
    static void New(HI_ARGS_INFO){\
        HI_SCOPE;\
        if (info.IsConstructCall()) {\
            CLS* obj = new CLS(info);\
            obj->Wrap(info.This());\
            info.GetReturnValue().Set(info.This());\
        }\
    }\
    static Local<Function> Constructor() {\
        HI_ISOLATE;\
        if (CLS::Template.IsEmpty()){\
            Local<FunctionTemplate> tpl = FunctionTemplate::New(isolate, New);\
            tpl->SetClassName(HI_STRING(#CLS));\
            tpl->InstanceTemplate()->SetInternalFieldCount(1);\
            CLS::Template.Reset(isolate, tpl);\
            __Define_Template(tpl);\
        }\
        return HI_CLASS_TPL(CLS)->GetFunction();\
    }\
    static void __Define_Template(Local<FunctionTemplate> tpl);\

// -> HI_CLASS_DEFINE_INIT
#define HI_CLASS_INIT(CLS)\
Persistent<FunctionTemplate> CLS::Template;\
CLS::CLS(HI_ARGS_INFO)

// -> HI_CLASS_DEFINE_PROPERTY
#define HI_CLASS_DEFINE_TEMPLATE(CLS)\
void CLS::__Define_Template(Local<FunctionTemplate> tpl)

// -> HI_CLASS_TPL
#define HI_CLASS_TPL(CLS)\
CLS::Template.Get(isolate)

// -> HI_CLASS_CONSTRUCTOR
#define HI_CLASS_CONSTRUCTOR(CLS)\
CLS::Constructor()

// -> HI_CLASS_DEFINE_METHOD
#define HI_CLASS_DEFINE_METHOD(CLS, fn)\
inline Local<Value> _##fn(HI_ARGS_INFO);\
static void fn(HI_ARGS_INFO){\
    CLS *_this = HI_UNWRAP(CLS, info.This());\
    HI_RETURN(_this->_##fn(info));\
}\

// -> HI_CLASS_METHOD
#define HI_CLASS_METHOD(CLS, name)\
Local<Value> CLS::_##name(HI_ARGS_INFO)

// -> HI_CLASS_DEFINE_GETTER
#define HI_CLASS_DEFINE_GETTER(CLS, name) \
inline Local<Value> _##name(Local<String> property, const PropertyCallbackInfo<Value>& info);\
static void name(Local<String> property, const PropertyCallbackInfo<Value>& info){\
    CLS *_this = HI_UNWRAP(CLS, info.This());\
    HI_RETURN(_this->_##name(property, info));\
}

// -> HI_CLASS_GETTER
#define HI_CLASS_GETTER(CLS, name) \
Local<Value> CLS::_##name(Local<String> property, const PropertyCallbackInfo<Value>& info)

// -> HI_CLASS_DEFINE_SETTER
#define HI_CLASS_DEFINE_SETTER(CLS, name) \
inline Local<Value> _##name(Local<String> property, Local<Value> value, const PropertyCallbackInfo<void>& info);\
static void name(Local<String> property, const PropertyCallbackInfo<Value>& info){\
    CLS *_this = HI_UNWRAP(CLS, info.This());\
    HI_RETURN(_this->_##name(property, info));\
}

// -> HI_CLASS_SETTER
#define HI_CLASS_SETTER(OBJ, name) \
Local<Value> _##name(Local<String> property, Local<Value> value, const PropertyCallbackInfo<void>& info);

// -> HI_CLASS_NEW
#define HI_CLASS_NEW_INSTANCE(CLS, argc, args)\
HI_NEW_INSTANCE(HI_CLASS_CONSTRUCTOR(CLS), argc, args)


// -> HI_INSTANCE_OF_CLASS
#define HI_INSTANCE_OF_CLASS(obj, CLS) \
HI_CLASS_TPL(CLS)->HasInstance(obj)


////////////////////////////////////////////////////////////////////////////
// -> HI_OBJ_DEFINE
#define HI_OBJ_DEFINE(OBJ)\
Persistent<FunctionTemplate> OBJ;\
void OBJ##_Init(HI_ARGS_INFO);\
void OBJ##_Define_Template(Local<FunctionTemplate> tpl);\
Local<Function> OBJ##_CONSTRUCTOR() {\
    HI_ISOLATE;\
    if (OBJ.IsEmpty()){\
        Local<FunctionTemplate> tpl = FunctionTemplate::New(isolate, OBJ##_Init);\
        tpl->SetClassName(HI_STRING(#OBJ));\
        tpl->InstanceTemplate()->SetInternalFieldCount(1);\
        OBJ.Reset(isolate, tpl);\
        OBJ##_Define_Template(tpl);\
    }\
    return OBJ.Get(isolate)->GetFunction();\
}\

// -> HI_OBJ_INIT
#define HI_OBJ_INIT(OBJ)\
void OBJ##_Init(HI_ARGS_INFO)

// -> HI_OBJ_DEFINE_TEMPLATE
#define HI_OBJ_DEFINE_TEMPLATE(OBJ)\
void OBJ##_Define_Template(Local<FunctionTemplate> tpl)

// -> HI_OBJ_CONSTRUCTOR
#define HI_OBJ_CONSTRUCTOR(OBJ) \
OBJ##_CONSTRUCTOR()

// -> HI_OBJ_METHOD
#define HI_OBJ_METHOD(OBJ, name) \
void OBJ##_##name(HI_ARGS_INFO)

// -> HI_OBJ_GETTER
#define HI_OBJ_GETTER(OBJ, name) \
void OBJ##_##name(Local<String> property, const PropertyCallbackInfo<Value>& info)

// -> HI_OBJ_SETTER
#define HI_OBJ_SETTER(OBJ, name) \
void OBJ##_##name(Local<String> property, Local<Value> value, const PropertyCallbackInfo<void>& info)

// -> HI_OBJ_METHOD_NAME
#define HI_OBJ_METHOD_NAME(OBJ, name) \
OBJ##_##name

// -> HI_OBJ_NEW_INSTANCE
#define HI_OBJ_NEW_INSTANCE(OBJ, argc, args)\
HI_NEW_INSTANCE(HI_OBJ_CONSTRUCTOR(OBJ), argc, args)

#define HI_OBJ_TPL(OBJ)\
OBJ.Get(isolate)

#define HI_INSTANCE_OF_OBJ(obj, OBJ) \
HI_OBJ_TPL(OBJ)->HasInstance(obj)

////////////////////////////////////////////////////////////////////////////
// -> HI_OBJECT_UNWRAP
#define HI_OBJECT_UNWRAP(OBJ)\
HI_ISOLATE;\
Local<Object> _this_obj = info.This();\
OBJ *_this = HI_UNWRAP(OBJ, _this_obj)

// -> HI_OBJECT_UNWRAP_THIS
#define HI_OBJECT_UNWRAP_THIS(OBJ)\
OBJ *_this = HI_UNWRAP(OBJ, info.This())

// -> HI_SCOPE_OBJECT_UNWRAP
#define HI_SCOPE_OBJECT_UNWRAP(OBJ)\
HI_SCOPE;\
Local<Object> _this_obj = info.This();\
OBJ *_this = HI_UNWRAP(OBJ, _this_obj)

////////////////////////////////////////////////////////////////////////////

// -> HI_TEMPLATE
#define HI_TEMPLATE(TPL)\
TPL.Get(isolate)

// -> HI_TEMPLATE_PROTOTYPE
#define HI_TEMPLATE_PROTOTYPE(tpl) \
tpl->GetFunction()->Get(HI_STRING("prototype")).As<Object>()

// -> HI_TEMPLATE_NEW_INSTANCE
#define HI_TEMPLATE_NEW_INSTANCE(tpl, argc, args) \
HI_NEW_INSTANCE(tpl->GetFunction(), argc, args)

// -> HI_NEW_INSTANCE
#define HI_NEW_INSTANCE(func, argc, args) \
func->NewInstance(isolate->GetCurrentContext(), argc, args).ToLocalChecked();

////////////////////////////////////////////////////////////////////////////

#define HI_NEW_PROMISE(V) \
Local<Promise::Resolver> V = Promise::Resolver::New(isolate)

#define HI_RETURN_PROMISE(V) \
Local<Promise::Resolver> V = Promise::Resolver::New(isolate);\
info.GetReturnValue().Set(V)


namespace HI {
    
    inline Local<String> ToString(const char *str) {
        return String::NewFromUtf8(Isolate::GetCurrent(), str);
    }
    
    inline Local<String> ToString(const string &str) {
        return String::NewFromUtf8(Isolate::GetCurrent(), str.c_str());
    }
    
    inline string ToString(const Local<Value>& value) {
        return string(*String::Utf8Value(value));
    }
    
////////////////////////////////////////////////////////////////////////////
    
    inline void SetPrototype(Local<FunctionTemplate> tpl, const char* name, FunctionCallback callback, bool readonly = false, bool dont_enum = false, bool dont_delete = false) {
        HI_SCOPE;
        Local<String> fn_name = HI_STRING(name);
        
        Local<FunctionTemplate> fn = v8::FunctionTemplate::New(isolate, callback, Local<Value>(), Signature::New(isolate, tpl));
        fn->SetClassName(fn_name);
    
        if (readonly || dont_enum || dont_delete) {
            Local<Object> prototype = HI_TEMPLATE_PROTOTYPE(tpl);
            PropertyDescriptor desc(fn->GetFunction(), !readonly);
            desc.set_enumerable(!dont_enum);
            desc.set_configurable(!dont_delete);
            prototype->DefineProperty(isolate->GetCurrentContext(), fn_name, desc).IsNothing();
        }else{
            tpl->PrototypeTemplate()->Set(fn_name, fn);
        }
    }
    
    inline void SetPrototype(Local<FunctionTemplate> tpl, const char* name, AccessorGetterCallback getter, AccessorSetterCallback setter = 0, bool readonly = false, bool dont_enum = false, bool dont_delete = false) {
        HI_SCOPE;
        Local<String> fn_name = HI_STRING(name);
        Local<Value> data = Local<Value>();
        AccessControl settings = DEFAULT;
        PropertyAttribute attribute = (readonly || dont_enum || dont_delete) ? static_cast<PropertyAttribute>((readonly ? ReadOnly : None) | (dont_enum ? DontEnum : None) | (dont_delete ? DontDelete : None)) : None;
        Local<AccessorSignature> signature = AccessorSignature::New(isolate, tpl);
        tpl->InstanceTemplate()->SetAccessor(fn_name, getter, setter, data, settings, attribute, signature);
    }
    
    inline void SetPrototype(Local<FunctionTemplate> tpl, const char* name, AccessorNameGetterCallback getter, AccessorNameSetterCallback setter = 0, bool readonly = false, bool dont_enum = false, bool dont_delete = false) {
        HI_SCOPE;
        Local<String> fn_name = HI_STRING(name);
        Local<Value> data = Local<Value>();
        AccessControl settings = DEFAULT;
        PropertyAttribute attribute = (readonly || dont_enum || dont_delete) ? static_cast<PropertyAttribute>((readonly ? ReadOnly : None) | (dont_enum ? DontEnum : None) | (dont_delete ? DontDelete : None)) : None;
        Local<AccessorSignature> signature = AccessorSignature::New(isolate, tpl);
        tpl->InstanceTemplate()->SetAccessor(fn_name, getter, setter, data, settings, attribute, signature);
        
    }
    
    inline void SetProperty(const Local<Object>& obj, const char* key, const Local<Value>& value, bool readonly = true, bool dont_enum = true, bool dont_delete = true) {
        PropertyDescriptor desc(value, !readonly);
        desc.set_enumerable(!dont_enum);
        desc.set_configurable(!dont_delete);
        obj->DefineProperty(Isolate::GetCurrent()->GetCurrentContext(), HI_STRING(key), desc).IsNothing();
    }

////////////////////////////////////////////////////////////////////////////

    inline Local<String> Json_stringify(const Local<Value>& val) {
        if(val->IsNumber() || val->IsNull() || val->IsUndefined() || val->IsBoolean()){
            return val->ToString();
        }
        if(val->IsString()){
            string str = string( *String::Utf8Value(val) );
            string res = "\"";
            for(string::iterator it = str.begin(); it != str.end(); ++it) {
                if(*it == '"' || *it == '\\'){
                    res += "\\";
                }
                res += *it;
            }
            res.append("\"");
            return Nan::New(res).ToLocalChecked();
        }

        if (val->IsObject()) {
            Nan::JSON NanJSON;
            Nan::MaybeLocal<String> result = NanJSON.Stringify(val.As<Object>());
            return result.ToLocalChecked();
        }
        return  HI_STRING("");
    }
    
    inline Local<Value> Json_parse(const string& str) {
        if (str == "null") {
            return Nan::Null();
        }
        if (str == "undefined") {
            return Nan::Undefined();
        }
        if (str == "true") {
            return Nan::True();
        }
        if (str == "false") {
            return Nan::False();
        }
        Nan::JSON NanJSON;
        return NanJSON.Parse(Nan::New(str).ToLocalChecked()).ToLocalChecked();
    }
    
////////////////////////////////////////////////////////////////////////////
    
    inline string Obj_Get(const Local<Object>& obj, const char *key, const char *def) {
        if (obj.IsEmpty()){
            return def;
        }
        Local<String> str = ToString(key);
        return obj->Has(str) ? HI_STR(obj->Get(str)) : def;
    }
    
    inline bool Obj_Get(const Local<Object>& obj, const char *key, bool def) {
        if (obj.IsEmpty()){
            return def;
        }
        Local<String> str = ToString(key);
        return obj->Has(str) ? obj->Get(str)->BooleanValue() : def;
    }
    
    inline int32_t Obj_Get(const Local<Object>& obj, const char *key, int32_t def) {
        if (obj.IsEmpty()){
            return def;
        }
        Local<String> str = ToString(key);
        return obj->Has(str) ? obj->Get(str)->Int32Value() : def;
    }
    
    
    
//    inline string std_string(const Local<Value>& val) {
//        if(val->IsNull() || val->IsUndefined()){
//            return string("");
//        }
//        if(val->IsString()){
//            string( *String::Utf8Value(val) );
//        }
//        return string( *String::Utf8Value(val->ToString()) );
//    }

}

#endif /* addon_utils_h */
