
#ifndef ldb_h
#define ldb_h

#include "hi-node.h"

#include "leveldb/db.h"
#include "leveldb/cache.h"
#include "leveldb/filter_policy.h"

#include "event_emitter.h"
#include "ldb_batch.h"
#include "ldb_async.h"

namespace Ldb {
    
    class HI_CLASS(Ldb) {
    private:
        uint32_t _state_code;
        EventEmitter *_event;
        leveldb::Options *_options;
        leveldb::WriteOptions _writeOpt;
        leveldb::ReadOptions _readOpt;
        string _location;
        
    public:
        leveldb::DB *leveldb;
        
        ~Ldb();
        
        bool IsOpen();
        
        leveldb::Status Open();
        
        leveldb::Status Put(const string& key, const string& value);
        
        leveldb::Status Get(const string& key, string& value);
        
        leveldb::Status Del(const string& key);
        
        leveldb::Status PutBatch (Batch* batch);
        
        inline leveldb::Status PutBatch (const Local<Value>& value);
        
        void Close();
        
        bool SetOptions(HI_ARGS_INFO, const bool check_error = false);
        
        void EmitError(const string& from, leveldb::Status& status, const string& key = "", const string& value = "");
        
        void EmitEvent(const string& from, const string& msg = "", const string& key = "", const string& value = "");
        
        static bool IsExist(const string& localtion, const leveldb::Options& opt = leveldb::Options());
        
        static leveldb::Options* TranslationOptions(const Local<Object>& opt_obj, leveldb::Options* opt);
        
        ///////////////////////////////////////////////
        
        HI_CLASS_DEFINE(Ldb);
        
        static HI_GETTER(jsGetterLocation);
        static HI_GETTER(jsGetterIsOpen);
        static HI_GETTER(jsGetterStateCode);
        
        static HI_METHOD(jsOpen);
        static HI_METHOD(jsClose);
        static HI_METHOD(jsPut);
        static HI_METHOD(jsGet);
        static HI_METHOD(jsDel);
        static HI_METHOD(jsForEach);
        static HI_METHOD(jsGetIterator);
        static HI_METHOD(jsAsyncPut);
        static HI_METHOD(jsAsyncGet);
        static HI_METHOD(jsAsyncDel);
        static HI_METHOD(jsAsyncForEach);
        static HI_METHOD(jsAddListener);
        static HI_METHOD(jsRemoveListener);
        
        static HI_METHOD(jsMatch);
        
    };
    
}

#endif /* ldb_h */





















