
#ifndef ldb_async_h
#define ldb_async_h

#include <string.h>
#include <list>
#include "ldb.h"
#include "ldb_iterator.h"
#include "leveldb/db.h"

#define PromiseObj Local<Promise::Resolver>&

namespace Ldb {
    
    class PromiseWorker : public Nan::AsyncWorker {
    public:
        Ldb* ldb;
        leveldb::Status status;
        PromiseWorker(Ldb* ldb, PromiseObj promise);
        PromiseWorker(Ldb* ldb, PromiseObj promise, Nan::Callback* callback);
        Local<Promise::Resolver> GetPromise();
        virtual void HandleOKCallback();
        virtual void HandleErrorCallback();
    };
    
    class GetWorker : public PromiseWorker {
    private:
        string _key;
        string _result;
    public:
        GetWorker (Ldb* ldb, PromiseObj promise, const string& key);
        virtual void Execute ();
        virtual void HandleOKCallback();
    };
    
    class GetListWorker : public PromiseWorker {
    private:
        list<string>* _keys;
        list<Data> _results;
    public:
        GetListWorker (Ldb* ldb, PromiseObj promise, list<string>* keys);
        virtual ~GetListWorker();
        virtual void Execute();
        virtual void HandleOKCallback();
    };
    
    class BatchWorker : public PromiseWorker {
    private:
        Batch* _batch;
    public:
        BatchWorker (Ldb* ldb, PromiseObj promise, Batch* batch);
        ~BatchWorker();
        virtual void Execute ();
    };
    
    class ForEachWorker: public Nan::AsyncWorker {
    private:
        Iterator *_iter;
    public:
        ForEachWorker (Ldb* ldb, PromiseObj promise, Iterator *iter);
        ForEachWorker (ForEachWorker* work);
        void Release();
        virtual void Execute ();
        virtual void HandleOKCallback();
        virtual void HandleErrorCallback();
    };
    
    
    class IteratoWorker: public PromiseWorker {
    public:
        enum Operate {
            Delete, Read
        };
    private:
        Iterator *_iter;
        Operate _operate;
    public:
        IteratoWorker (Ldb* ldb, PromiseObj promise, Iterator *iter, Operate operate);
        virtual void Execute ();
        virtual void HandleOKCallback();
    };
        
}

#endif /* ldb_h */

