
#include "ldb_batch.h"

//#include <iostream>

namespace Ldb {

    void Batch::Add(const Local<Value>& data) {
        if (data->IsArray()){
            this->AddByArray(data.As<Array>());
        }else if(data->IsObject()){
            this->AddByObject(data.As<Object>());
        }
    }
    
    void Batch::AddByArray(const Local<Array>& names) {
        
        HI_SCOPE;
        for (unsigned int i = 0; i < names->Length(); i++) {
            Local<Value> v = names->Get(i);
            if (v->IsObject()){
                Local<Object> opt = v.As<Object>();
                string key = HI::Obj_Get(opt, "key", "");
                if (key.empty()){
                    continue;
                }
                string type = HI::Obj_Get(opt, "type", "put");
                if(type == "put"){
                    string value = HI_TO_STR(HI::Json_stringify(opt->Get(HI_STRING("value"))));
                    bool append = HI::Obj_Get(opt, "append", false);
                    this->Put(key, value, append);
                }else if(type == "del"){
                    this->Del(key);
                }
            }else{
                this->Del( HI_TO_STR(names->Get(i)) );
            }
        }
    }
    
    void Batch::AddByObject(const Local<Object>& obj) {
        Local<Array> names = obj->GetPropertyNames();
        for (unsigned int i = 0; i < names->Length(); i++) {
            Local<Value> name = names->Get(i);
            Local<Value> value = obj->Get(name);
            string key = HI_TO_STR(name);
            if (value->IsNull() || value->IsUndefined()) {
                this->Del(key);
            }else{
                string text = HI_TO_STR(HI::Json_stringify(value));
                this->Put(key, text);
            }
        }
    }
    

    void Batch::Put(const string& key, const string& value, const bool append) {
        this->datas.push_back(Data(append ? Data::Append : Data::Put, key, value));
    }

    void Batch::Del(const string& key) {
        this->datas.push_back(Data(Data::Del, key, ""));
    }

    leveldb::WriteBatch* Batch::Export() {
        leveldb::WriteBatch* basth = new leveldb::WriteBatch();
        for(list<Data>::iterator it = this->datas.begin(); it != this->datas.end(); it++){
            if (it->type == Data::Del) {
                basth->Delete(it->key);
            }else if (it->type == Data::Put){
                basth->Put(it->key, it->value);
            }
        }
        return basth;
    }
    
}

