
#include "ldb.h"

//#include <iostream>

namespace ldb_module {
    
    HI_METHOD(OpenDB){
        HI_SCOPE;
        if (info.Length() < 1) {
            return Nan::ThrowError("[Ldb.open error] 1 argument required, but only 0 present");
        }
        Local<Object> ldb_obj = HI_CLASS_NEW_INSTANCE(Ldb::Ldb, 0, NULL);
        Ldb::Ldb *ldb = HI_UNWRAP(Ldb::Ldb, ldb_obj);
        if(ldb->SetOptions(info, true)) {
            if (ldb->Open().ok()) {
                HI_RETURN(ldb_obj);
            }
        }
    }
    
    HI_METHOD(DestroyDB) {
        HI_SCOPE;
        string location = HI_STR(info[0]);
        if (!location.empty()) {
            leveldb::Options options;
            leveldb::Status status = leveldb::DestroyDB(location, options);
            HI_RETURN(HI_BOOLEAN(status.ok()));
        }
    }
    
    HI_METHOD(RepairDB) {
        HI_SCOPE;
        string location = HI_STR(info[0]);
        if (!location.empty()) {
            leveldb::Options options;
            leveldb::Status status = leveldb::RepairDB(location, options);
            HI_RETURN(HI_BOOLEAN(status.ok()));
        }
    }
    
    HI_METHOD(jsIsExist) {
        HI_SCOPE;
        string location = HI_STR(info[0]);
        if (!location.empty()) {
            HI_RETURN(HI_BOOLEAN( Ldb::Ldb::IsExist(location) ));
        }
    }
    
    void Init(Handle<Object> exports, Handle<Object> module) {
        HI_SCOPE;
        Local<Function> fn = HI_CLASS_CONSTRUCTOR(Ldb::Ldb);
        fn->Set(HI_STRING("open"), HI_FUNCTION(OpenDB));
        fn->Set(HI_STRING("destroy"), HI_FUNCTION(DestroyDB));
        fn->Set(HI_STRING("repair"), HI_FUNCTION(RepairDB));
        fn->Set(HI_STRING("isExist"), HI_FUNCTION(jsIsExist));
        module->Set(HI_STRING("exports"), fn);
    }
    
}

NODE_MODULE(Ldb, ldb_module::Init);

